#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#


import numpy as np
import math
import random
import matplotlib.pyplot as plt
import matplotlib.transforms as transforms
from astropy.io import fits
import argparse
import sys
import csv
from lmfit import minimize, Parameters

from .analyze import axvlines
from .analyze import gaussfunc, gaussfunc_gh, gauss2func
from .analyze import linfunc


def main(args=None):
# Parser
    parser = argparse.ArgumentParser(description='ANALYZE SPECTRUM',prog='analyze_spectrum')
    parser.add_argument('-s', '--spectrum', metavar='SPECTRUM/FILE_LIST', help='FITS spectrum / list of FITS spectra', type=argparse.FileType('rb'))
    parser.add_argument('-l', '--is-a-list', default=False, action="store_true", help='Use for -s being a list of FITS spectra')
    parser.add_argument('-f', '--method', default=0, choices=[0,1,2], metavar='FITTING FUNCTION (0,1,2)', help='Fitting function (0=gauss_hermite, 1=gauss, 2=double_gauss)', type=int)

    parser.add_argument('-w', '--ctwl', metavar='LINE CENTRAL WAVELENGTH', help='Central rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-k', '--use-peak', default=False, action="store_true", help='Use peak first guess on central wavelength')
    parser.add_argument('-LW1', '--lcut1', metavar='LOWER WAVELENGTH - LINE', help='Lower rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-LW2', '--lcut2', metavar='UPPER WAVELENGTH - LINE', help='Upper rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-CW1', '--ccut1', metavar='LOWER WAVELENGTH - CONT', help='Lower rest-frame wavelength for cont. (AA)',
                        type=float)
    parser.add_argument('-CW2', '--ccut2', metavar='UPPER WAVELENGTH - CONT', help='Upper rest-frame wavelength for cont. (AA)',
                        type=float)
    parser.add_argument('-ECW1', '--eccut1', metavar='EXCLUDE FROM CONT. (LOWER WAVELENGTH)', help='Lower rest-frame wavelength of range to exclude for cont. (AA)',
                        type=float)
    parser.add_argument('-ECW2', '--eccut2', metavar='EXCLUDE FROM CONT. (UPPER WAVELENGTH)', help='Upper rest-frame wavelength of range to exclude for cont. (AA)',
                        type=float)
    parser.add_argument('-PW1', '--pcut1', metavar='LOWER WAVELENGTH - PLOT', help='Lower rest-frame wavelength for plot (AA)',
                        type=float)
    parser.add_argument('-PW2', '--pcut2', metavar='UPPER WAVELENGTH - PLOT', help='Upper rest-frame wavelength for plot (AA)',
                        type=float)
    parser.add_argument('-S2', '--scale-amp2', metavar='SCALE FACTOR FOR AMP2', help='Scale factor for amplitude 2', default=1.0, type=float)
    parser.add_argument('-t', '--spec-table', metavar='SPEC-TABLE', help='Additional spectrum table', type=argparse.FileType('r'))
    parser.add_argument('-c', '--catalog', metavar='LINECAT-TABLE', help='Cataloged lines CSV table', type=argparse.FileType('r'))
    parser.add_argument('-z', '--redshift', metavar='REDSHIFT', help='Redshift for target and catalog lines', type=float)
    parser.add_argument('-o', '--output', metavar='OUTPUT-PDF', help='Output PDF', type=argparse.FileType('w'))
    parser.add_argument('-p', '--plot', default=False, action="store_true" , help='Plot spectrum?')
    parser.add_argument('-n', '--no-legend', default=False, action="store_true", help='Legend?')

    args = parser.parse_args(args=args)
    
# Constants
    c_amstrong = 2.99792e+18  # Light speed in AA/s
    two = 2
    zero = np.float64(0.0)
    
    if args.redshift is not None:
       z=float(args.redshift)
    else:
       z=0.

# Plotting
    if args.spec_table!=None or args.spectrum!=None:
       plt.figure()

    if args.spectrum!=None:   
# Reading spectrum/spectra
       if args.is_a_list == True:
          list_files = args.spectrum.readlines() 
          list_files = [str(x.strip(),'utf-8') for x in list_files] 
       else: 
          list_files = [args.spectrum.name]
          
       for ifile in list_files:
          hdulist = fits.open(ifile)
          tbdata = hdulist[0].data
          prihdr = hdulist[0].header
          lambda0 = prihdr['CRVAL1']
          cdelt = prihdr['CDELT1']
          crpix = prihdr['CRPIX1']
          if ('VPH' in prihdr):
             vph = prihdr['VPH']
          else:
             vph = 'LR'
 
          lambda_fin = lambda0 + (len(tbdata))*cdelt
       
          if 'BUNIT' in prihdr:
             bunit = prihdr['BUNIT']
          else:
             bunit = 'Jy' # To take into account that the QLA-generated 1D spectra have not BUNIT keyword
                
          if 'PIXLIMF1' in prihdr:
             pf1 = prihdr['PIXLIMF1'] # Sensitivity function computed in this region (beginning)
          else: 
             pf1 = crpix 
          if 'PIXLIMF2'in prihdr:
             pf2 = prihdr['PIXLIMF2'] # Sensitivity function computed in this region (end)
          else:
             pf2 = len(tbdata)
          if 'PIXLIMM1' in prihdr:
             pm1 = prihdr['PIXLIMM1'] # All fibers include this region (beginning)
          else:
             pm1 = crpix
          if 'PIXLIMM2' in prihdr:
             pm2 = prihdr['PIXLIMM2'] # All fibers include this region (end)
          else:
             pm2 = len(tbdata)
                
          flux = []
          wave = []
            
          for i in range(0,len(tbdata)):
              lambda_i = lambda0 + i*cdelt
              wave.append(lambda_i)
              if bunit == 'Jy':
                 flux_vector = (1e-23*tbdata[i]*c_amstrong)/(lambda_i**2)
              elif bunit == 'ELECTRON' or 'CGS' or 'cgs' in bunit:
                 flux_vector = tbdata[i]
              else:
                 print ('unknown or not defined BUNIT [Jy, ELECTRON]')
                 sys.exit(1)
              flux.append(flux_vector)
  
          plt.xlim(float(args.pcut1),float(args.pcut2))
        
          if "LR" in vph:
              plt.plot(wave, flux, 'blue', label = 'input spectrum')
              R=6000.0
          else: 
              if "MR" in vph:
                  plt.plot(wave, flux, 'green', label = 'input spectrum')
                  R=12000.0
              else:
                  plt.plot(wave, flux, 'red', label = 'input spectrum')
                  R=20000.0

          fcont = []
          wcont = []
          fcont = flux[int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.ccut2)*(1.+z)-lambda0)/cdelt+crpix)]
# Excluding range
          if (args.eccut1 is not None and args.eccut2 is not None):
              del fcont[int((float(args.eccut1)*(1.+z)-lambda0)/cdelt+crpix)-int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.eccut2)*(1.+z)-lambda0)/cdelt+crpix)-int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix)]
          wcont = wave[int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.ccut2)*(1.+z)-lambda0)/cdelt+crpix)]
          if (args.eccut1 is not None and args.eccut2 is not None):
              del wcont[int((float(args.eccut1)*(1.+z)-lambda0)/cdelt+crpix)-int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.eccut2)*(1.+z)-lambda0)/cdelt+crpix)-int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix)]
          cmean = np.mean(fcont)

# Fitting continuum
          p_lin = Parameters()
          p_lin.add('slope', value=zero, vary=True)
          p_lin.add('yord', value=cmean, vary=True)
          print("FITTING CONTINUUM:")
          print("Input(slope,yord):  %10.3E %10.3E" % (0., cmean))
          err_lin = lambda p, x, y: linfunc(p, x) - y
          fitout_lin=minimize(err_lin,p_lin,args=(wcont,fcont))
          fitted_p_lin = fitout_lin.params
          pars_lin = [fitout_lin.params['slope'].value, fitout_lin.params['yord'].value,
                     fitout_lin.chisqr]
          print("Output(slope,yord): %10.3E %10.3E"%(fitout_lin.params['slope'].value, fitout_lin.params['yord'].value))
          print("Best-fitting chisqr continuum: %10.3E" % (fitout_lin.chisqr))
          fit_con = linfunc(fitted_p_lin, wcont)
          residuals = fcont - fit_con
          rms = np.std(residuals)

# Arrays for line profile
          fline = []
          fpline = []
          wline = []
          fline = flux[int((float(args.lcut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.lcut2)*(1.+z)-lambda0)/cdelt+crpix)]
          wline = wave[int((float(args.lcut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.lcut2)*(1.+z)-lambda0)/cdelt+crpix)]
          peak = np.amax(fline)
          fit_lin = linfunc(fitted_p_lin, wline)
          lcmean = np.mean(fit_lin) # Mean continuum within the line range
          fpline = fline - fit_lin
#          fpline = fline - cmean
          result = np.where(fline == np.amax(fline))
          lpeak = wline[result[0][0]]

          print ("BASIC NUMBERS:")
          print ("(mean,rms,lpk,pk,S/N)",lcmean,rms,lpeak,peak,peak/rms)

# Initial guess on parameters

          amp = peak-lcmean
          if (args.use_peak) == False:
            center = float(args.ctwl)*(1.+z)
          else:
            center = lpeak
          sigma = (float(args.ctwl)/R)*((1.+z)/2.35)
          skew = 0.0
          kurt = 0.0
          
          amp1 = 0.9*(peak-lcmean)
          sigma1 = (float(args.ctwl)/R)/2.35
          amp2 = 0.1*(peak-lcmean)
          sigma2 = 2.*(float(args.ctwl)/R)/2.35
          if (args.use_peak) == False:
            center1 = float(args.ctwl)*(1.+z)
            center2 = float(args.ctwl)*(1.+z)
          else:
            center1 = lpeak
            center2 = lpeak
                       
          if args.method is 0:
              p_gh=Parameters()
              p_gh.add('amp',value=amp, vary=True)
              p_gh.add('center',value=center, vary=True)
              p_gh.add('sigma',value=sigma, vary=True, min=0.8*sigma)
              p_gh.add('skew',value=skew,vary=True,min=-0.5,max=0.5)
              p_gh.add('kurt',value=kurt,vary=True,min=-0.5,max=0.5)
              print ("FITTING METHOD: GAUSS-HERMITE QUADRATURE")
              print ("Input(i0,l0,sigma,skew,kurt):  %10.3E %5.2f %5.2f %10.3E %10.3E"%(amp, center, sigma, skew, kurt))
              gausserr_gh = lambda p,x,y: gaussfunc_gh(p,x)-y 
          if args.method is 1:
              p_gh=Parameters()
              p_gh.add('amp',value=amp, vary=True)
              p_gh.add('center',value=center, vary=True)
              p_gh.add('sigma',value=1.2*sigma, vary=True, min=sigma)
              print ("FITTING METHOD: SINGLE GAUSSIAN")
              print ("Input(i0,l0,sigma):  %10.3E %5.2f %5.2f"%(amp, center, sigma))
              gausserr_gh = lambda p,x,y: gaussfunc(p,x)-y
          if args.method is 2:
              p_gh=Parameters()
              p_gh.add('amp1',value=amp1, vary=True);
              p_gh.add('center1',value=center1, vary=True);
              p_gh.add('sigma1',value=sigma1, vary=True, min=0.8*sigma1);
              p_gh.add('amp2',value=args.scale_amp2*amp2, vary=True);
              p_gh.add('center2',value=center2, vary=True);
              p_gh.add('sigma2',value=sigma2, vary=True, min=1.5*sigma1);
              print ("FITTING METHOD: DOUBLE GAUSSIAN")
              print ("Input(i1,l1,sig1,i2,l2,sig2):  %10.3E %5.2f %5.2f %10.3E %5.2f %5.2f"%(amp1, center1, sigma1, args.scale_amp2*amp2, center2, sigma2))
              gausserr_gh = lambda p,x,y: gauss2func(p,x)-y
                
          fitout_gh=minimize(gausserr_gh,p_gh,args=(wline,fpline))
          fitted_p_gh = fitout_gh.params
          tmp = fitout_gh
          if args.method is 2:
             tmp = fitted_p_gh
             tmp.add('amp',value=tmp['amp1'].value)
             tmp.add('center',value=tmp['center1'].value)
             tmp.add('sigma',value=tmp['sigma1'].value)
             fitted_p_gh1 = tmp
             fit_gh1=gaussfunc(fitted_p_gh1,wline)
             print ("Flux1 from model: %10.3E+/-%10.3E"%(cdelt*np.sum(fit_gh1), rms*cdelt*np.sqrt(2*len(fit_gh1)+(np.sum(fit_gh1)/lcmean)))) # Errors as in Tresse et al. (1999)
             tmp2 = fitted_p_gh
             tmp2.add('amp',value=tmp2['amp2'].value)
             tmp2.add('center',value=tmp2['center2'].value)
             tmp2.add('sigma',value=tmp2['sigma2'].value)
             fitted_p_gh2 = tmp2
             fit_gh2=gaussfunc(fitted_p_gh2,wline)
             print ("Flux2 from model: %10.3E+/-%10.3E"%(cdelt*np.sum(fit_gh2), rms*cdelt*np.sqrt(2*len(fit_gh2)+(np.sum(fit_gh2)/lcmean)))) # Errors as in Tresse et al. (1999)

          if args.method is 0:
              pars_gh=[fitout_gh.params['amp'].value,fitout_gh.params['center'].value,fitout_gh.params['sigma'].value,fitout_gh.params['skew'].value,fitout_gh.params['kurt'].value,fitout_gh.chisqr]
              fit_gh=gaussfunc_gh(fitted_p_gh,wline)
              print ("Output(i0,l0,sigma,skew,kurt): %10.3E %5.2f %5.2f %10.3E %10.3E"%(fitout_gh.params['amp'].value, fitout_gh.params['center'].value, fitout_gh.params['sigma'].value, fitout_gh.params['skew'].value, fitout_gh.params['kurt'].value))
          if args.method is 1:
              pars_gh=[fitout_gh.params['amp'].value,fitout_gh.params['center'].value,fitout_gh.params['sigma'].value,fitout_gh.chisqr]
              fit_gh=gaussfunc(fitted_p_gh,wline)
              print ("Output(i0,l0,sigma): %10.3E %5.2f %5.2f"%(fitout_gh.params['amp'].value, fitout_gh.params['center'].value, fitout_gh.params['sigma'].value))
          if args.method is 2:
              pars_gh=[fitout_gh.params['amp1'].value,fitout_gh.params['center1'].value,fitout_gh.params['sigma1'].value,fitout_gh.params['amp2'].value,fitout_gh.params['center2'].value,fitout_gh.params['sigma2'].value,fitout_gh.chisqr] 
              fit_gh=gauss2func(fitted_p_gh,wline)
              print ("Output(i1,l1,sig1,i2,l2,sig2): %10.3E %5.2f %5.2f %10.3E %5.2f %5.2f"%(fitout_gh.params['amp1'].value, fitout_gh.params['center1'].value, fitout_gh.params['sigma1'].value, fitout_gh.params['amp2'].value, fitout_gh.params['center2'].value, fitout_gh.params['sigma2'].value))
#          print ("CDELT1: %6.3f Angstroms/pixel"%(cdelt))
          eEWd=(rms*cdelt*(cdelt*np.sum(fpline)/lcmean)/(cdelt*np.sum(fpline)))*np.sqrt(2*len(fpline)+np.sum(fpline)/lcmean+(np.sum(fpline)/lcmean)**2/len(fpline))
          eEWm=(rms*cdelt*(cdelt*np.sum(fit_gh)/lcmean)/(cdelt*np.sum(fit_gh)))*np.sqrt(2*len(fit_gh)+np.sum(fit_gh)/lcmean+(np.sum(fpline)/lcmean)**2/len(fit_gh))
          print ("Flux & EW from data:  %10.3E+/-%10.3E %5.2f+/-%5.2f"%(cdelt*np.sum(fpline), rms*cdelt*np.sqrt(2*len(fpline)+(np.sum(fpline)/lcmean)), cdelt*np.sum(fpline)/lcmean, eEWd)) # Errors as in Tresse et al. (1999)
          print ("Flux & EW from model: %10.3E+/-%10.3E %5.2f+/-%5.2f"%(cdelt*np.sum(fit_gh), rms*cdelt*np.sqrt(2*len(fit_gh)+(np.sum(fit_gh)/lcmean)), cdelt*np.sum(fit_gh)/lcmean, eEWm)) # Errors as in Tresse et al. (1999)
          print ("Best-fitting chisqr: %10.3E"%(fitout_gh.chisqr))

# Here we should write the results to a file and decide whether we should set them first to zero if S/N is below some threshold

          plt.plot(wcont, fit_con, 'red', label = 'Continuum fit')
          resid_gh=fpline-fit_gh
          plt.plot(wline, fit_gh+fit_lin, 'orange', label = 'best fit')

          lflines = [lambda0 + (pf1 - crpix) * cdelt, lambda0 + (pf2 - crpix) * cdelt]
          lmlines = [lambda0 + (pm1 - crpix) * cdelt, lambda0 + (pm2 - crpix) * cdelt]
          lllines = [float(args.lcut1)*(1.+z),float(args.lcut2)*(1.+z)]
          lclines = [float(args.ccut1)*(1.+z),float(args.ccut2)*(1.+z)]
          leclines = [float(args.eccut1)*(1.+z),float(args.eccut2)*(1.+z)]
          cwline = [float(args.ctwl)*(1.+z)]
            
          if (args.no_legend) == False:
              axvlines(lmlines, color='cyan', label = 'All-fiber range', linestyle = '-')
              axvlines(lflines, color='brown', label = 'Sensitivity range', linestyle = '--')
              if (args.use_peak) == False:
                 axvlines(cwline, color='black', label = 'Central wavelength', linestyle = '-')
              axvlines(lllines, color='gray', label = 'Line-fitting range', linestyle = '-')
              axvlines(lclines, color='gray', label = 'Continuum range', linestyle = '--')
              axvlines(leclines, color='gray', linestyle = '--')

    if args.spec_table!=None:
# Reading standard-star spectrum table
       xdat, ydat = np.loadtxt(args.spec_table, skiprows=1, usecols=(0, 1), unpack=True)
       fydat = []
       for i in range(0,len(ydat)):
           flux_value = (1e-23*(3631.0*10.**(-0.4*ydat[i]))*c_amstrong)/(xdat[i]**2)
           fydat.append(flux_value)
       plt.plot(xdat, fydat, 'red', label = 'tabulated spectrum')
    
    if args.catalog!=None:
# Reading input line catalog
        with open(args.catalog.name, 'r') as csvfile:
            spamreader = csv.reader(csvfile, delimiter=',', quotechar='"')
            slines=[] # Wavelengths
            llines=[] # Labels
            ylines=[] # Y positions
            a = np.array(list(spamreader))
            for i in a[:,1]:
                slines.append(float(i)*(1+z))
            for i in a[:,0]:
                llines.append(str(i))
                ylines.append(random.uniform(0.9,0.98))
            axvlines(slines, color='green', label=None, linestyle = ':')
            trans = transforms.blended_transform_factory(plt.gca().transData, plt.gca().transAxes)
            for i in range(0,len(slines)):
                plt.text(slines[i],ylines[i],llines[i],transform=trans,fontsize='smaller')

# Plotting
    if args.spectrum!=None:
       if bunit == 'Jy' or 'CGS' or 'cgs' in bunit:
          plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ $\AA$$^{-1}$]')
       else:
          plt.ylabel('ELECTRON')
    elif args.spec_table!=None:
         plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ $\AA$$^{-1}$]')

    if args.spec_table!=None or args.spectrum!=None:
       plt.xlabel('wavelength')
       if args.no_legend == True:
          plt.legend('', frameon=False)
       else:
          plt.legend()
       if args.plot == True:
          plt.show()
       else:
          if args.output!=None:
             plt.savefig(args.output.name)
          else:
             plt.show()


if __name__ == '__main__':
    
    main()



#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#


import numpy as np

import matplotlib.pyplot as plt


def axvlines(xs, **plot_kwargs):
    xs = np.array((xs, ) if np.isscalar(xs) else xs, copy=False)
    lims = plt.gca().get_ylim()
    x_points = np.repeat(xs[:, None], repeats=3, axis=1).flatten()
    y_points = np.repeat(np.array(lims + (np.nan, ))[None, :], repeats=len(xs), axis=0).flatten()
    plot = plt.plot(x_points, y_points, scaley = False, **plot_kwargs)
    return plot


def gaussfunc(paramsin,x):
    amp=paramsin['amp'].value
    center=paramsin['center'].value
    sigma=paramsin['sigma'].value
    g1=(x-center)/sigma
    gaustot=amp*np.exp(-.5*g1**2)
    return gaustot


def gauss2func(paramsin,x):
    amp1=paramsin['amp1'].value
    amp2=paramsin['amp2'].value
    center1=paramsin['center1'].value
    center2=paramsin['center2'].value
    sigma1=paramsin['sigma1'].value
    sigma2=paramsin['sigma2'].value
    g1=(x-center1)/sigma1
    g2=(x-center2)/sigma2
    gaus1=amp1*np.exp(-.5*g1**2)
    gaus2=amp2*np.exp(-.5*g2**2)
    gaustot_2g=(gaus1+gaus2)
    return gaustot_2g


def gaussfunc_gh(paramsin,x):
    amp=paramsin['amp'].value
    center=paramsin['center'].value
    sigma=paramsin['sigma'].value
    c1=-np.sqrt(3)
    c2=-np.sqrt(6)
    c3=2/np.sqrt(3)
    c4=np.sqrt(6)/3
    c5=np.sqrt(6)/4
    skew=paramsin['skew'].value
    kurt=paramsin['kurt'].value
    g1=(x-center)/sigma
    gaustot_gh=amp*np.exp(-.5*g1**2)*(1+skew*(c1*g1+c3*g1**3)+kurt*(c5+c2*g1**2+c4*(g1**4)))
    return gaustot_gh


def linfunc(paramsin,x):
    slope=paramsin['slope'].value
    yord=paramsin['yord'].value
    lintot=slope*np.array(x)+yord
    return lintot


#
# Copyright 2017-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

import math
from pathlib import Path
import numpy as np
from astropy.io import fits

import megaradrp.processing.cube as mcube

def trim_cubes(cube_file, trim_numbers):
    hdu = fits.open(cube_file)
    data = hdu[0].data
    for i in range(trim_numbers[0]):
        data = np.delete(data,0,1)   # Corto filas de pixeles de abajo.
    for i in range(trim_numbers[1]):
        data = np.delete(data,-1,1)  # Corto files de pixeles de arriba.
    for i in range(trim_numbers[2]):
        data = np.delete(data,0,2)   # Corto columnas de pixeles de la izquierda.
    for i in range(trim_numbers[3]):
        data = np.delete(data,-1,2)  # Corto columnas de pixeles de la derecha.
    hdu[0].header['NAXIS1'] = hdu[0].header['NAXIS1'] - 2
    hdu[0].header['NAXIS2'] = hdu[0].header['NAXIS2'] - 3
    hdu[0].header['CRPIX1'] = hdu[0].header['CRPIX1'] - 1
    hdu[0].header['CRPIX2'] = hdu[0].header['CRPIX2'] - 1
    hdu[0].data = data
    path = Path(cube_file)
    new_path = path.parent / ('trimmed_' + path.name)
    hdu.writeto(str(new_path), overwrite=True)

def helio_corr(ifile):
    from astropy.time import Time
    from astropy.coordinates import SkyCoord, EarthLocation
    import astropy.units as u
    hdu = fits.open(ifile)
    h1_0 = hdu[0].header

    obs_date = h1_0['DATE-OBS'].split('T')[0]

    roque = EarthLocation.from_geodetic(lat=28.7606*u.deg, lon=342.1184*u.deg, height=2326*u.m)
    sc = SkyCoord(ra=h1_0['RADEG']*u.deg, dec=h1_0['DECDEG']*u.deg)
    heliocorr = sc.radial_velocity_correction('heliocentric', obstime=Time(obs_date), location=roque)  
    helio_corr_velocity = heliocorr.to(u.km/u.s).value

    return helio_corr_velocity

def hypercube_dimensions(list_cubes, xoff, yoff, target_scale):
    RA_list = []
    DEC_list = []
    for i, ifile in enumerate(list_cubes):
        hdu = fits.open(ifile)
        RA_list.append(hdu[0].header['RADEG'] + (xoff[i]/3600))
        DEC_list.append(hdu[0].header['DECDEG'] + (yoff[i])/3600)
    pixel_size_x = target_scale/np.cos(hdu[0].header['DECDEG']*np.pi/180) # Este es el que aparece al medir en la imagen # arcsec
    pixel_size_y = target_scale # arcsec sin tener en cuenta posicion del telescopio
   
    ref_data = fits.open(list_cubes[0])[0].data
    data_size_x = ref_data.shape[2]
    data_size_y = ref_data.shape[1]  
    data_size_z = ref_data.shape[0]  # Direccion espectral

    hypercube_size_x = int(round((abs(max(RA_list)-min(RA_list))*3600/pixel_size_x) + data_size_x)) # De centro a centro + otro apuntado entero 
    hypercube_size_y = int(round((abs(max(DEC_list)-min(DEC_list))*3600/pixel_size_y) + data_size_y)) # De centro a centro + otro apuntado entero 
    hypercube_size_z = data_size_z
    return hypercube_size_x, hypercube_size_y, hypercube_size_z, RA_list, DEC_list, pixel_size_x, pixel_size_y, data_size_x, data_size_y

def to_bool(s):
    return True if s == '1' else False

def mask_bin(list_cubes, outfile, xoff, yoff, target_scale):
    hypercube_size_x, hypercube_size_y, hypercube_size_z, RA_list, DEC_list, pixel_size_x, pixel_size_y, data_size_x, data_size_y = hypercube_dimensions(list_cubes, xoff, yoff, target_scale)
    hdu = fits.open(list_cubes[0])
    mask = np.zeros((hypercube_size_y,hypercube_size_x), int)
    for i, ifile in enumerate(list_cubes):
        offset_pixels_x = int(round((abs((RA_list[i]-max(RA_list))*3600/pixel_size_x))))
        offset_pixels_y = int(round((abs((DEC_list[i]-min(DEC_list))*3600/pixel_size_y))))        
        for j in range(data_size_y):
            for k in range(data_size_x):
                mask[j+offset_pixels_y,k+offset_pixels_x] += 2**(i) 
    hdu[0].data = mask
    hdu[0].header['NAXIS'] = 2
    hdu[0].header['NAXIS1'] = mask.shape[1]
    hdu[0].header['NAXIS2'] = mask.shape[0]    
    path = Path(outfile)
    new_path = path.parent / (path.stem + '_mask' + path.suffix)
    hdu.writeto(str(new_path), overwrite=True)
#    hdu.writeto(str(outfile + '_hypercube_mask.fits'), overwrite=True)
    boolean_mask = np.empty(mask.shape,dtype=object)
    for j in range(mask.shape[0]):
        for k in range(mask.shape[1]):
            b = bin(mask[j,k])[2:].zfill(len(list_cubes))
            b = list(b)
            b = b[::-1]
            a = []
            for i,val in enumerate(b):
                a.append(to_bool(val))
            boolean_mask[j,k] = np.array(a)
    
    return mask, boolean_mask

def rebin_spec(wave, specin, wavnew):
    from pysynphot import observation
    from pysynphot import spectrum

    spec = spectrum.ArraySourceSpectrum(wave=wave, flux=specin)
    f = np.ones(len(wave))
    filt = spectrum.ArraySpectralElement(wave, f, waveunits='angstrom')
    obs = observation.Observation(spec, filt, binset=wavnew, force='taper')
 
    return obs.binflux

def grid_combined_cube(list_cubes, helio_corr_apply, outfile, xoff, yoff, scale_a, scale_m, target_scale):
    from itertools import compress
    from astropy.time import Time
    from astropy.coordinates import SkyCoord, EarthLocation
    import astropy.units as u

    hypercube_size_x, hypercube_size_y, hypercube_size_z, RA_list, DEC_list, pixel_size_x, pixel_size_y, data_size_x, data_size_y = hypercube_dimensions(list_cubes, xoff, yoff, target_scale)

    hypercube = np.zeros([hypercube_size_z,hypercube_size_y,hypercube_size_x])
    alldata = np.zeros((hypercube_size_z,hypercube_size_y,hypercube_size_x,len(list_cubes)), float)

    if helio_corr_apply:
    
        for i, ifile in enumerate(list_cubes):
            offset_pixels_x = int(round((abs((RA_list[i]-max(RA_list))*3600/pixel_size_x))))
            offset_pixels_y = int(round((abs((DEC_list[i]-min(DEC_list))*3600/pixel_size_y))))        
            hdu = fits.open(ifile)
            data_ifile = hdu[0].data
            
            h1_0 = hdu[0].header
            lambda_obs = h1_0['CRVAL3'] + h1_0['CDELT3']*np.arange(h1_0['NAXIS3'])
            lambda_em = np.zeros(data_ifile.shape)
            step = h1_0['CDELT3']
            
            helio_corr_velocity = helio_corr(ifile)
            print('Applying heliocentric correction to', ifile, '\n')
            print('Velocity correction:', helio_corr_velocity)
            
            c = 299792.458 # km/s
            
            z = np.exp(helio_corr_velocity/c) - 1
                
            lambda_em = lambda_obs/(1+z)
            
            # Esto es porque necesito saber todas las velocidades a las que voy a 
            # corregir los espectros para tener el rango completo de longitudes de onda. 
            
            lambda_completa = lambda_em.min() + step*np.arange(len(lambda_obs))
            
            spec_helio_corr = np.zeros((len(lambda_completa), data_ifile.shape[1], data_ifile.shape[2]))
            for j in range(data_ifile.shape[1]):
                for k in range(data_ifile.shape[2]):
                    spec_helio_corr[:,j,k] = rebin_spec(lambda_em, data_ifile[:,j,k], lambda_completa)
                    alldata[:,j+offset_pixels_y,k+offset_pixels_x,i] = spec_helio_corr[:,j,k]*scale_m[i] + scale_a[i]

    else:
        print('No heliocentric correction applied')
        for i, ifile in enumerate(list_cubes):
            offset_pixels_x = int(round((abs((RA_list[i]-max(RA_list))*3600/pixel_size_x))))
            offset_pixels_y = int(round((abs((DEC_list[i]-min(DEC_list))*3600/pixel_size_y))))        
            hdu = fits.open(ifile)
            data_ifile = hdu[0].data
            for j in range(data_size_y):
                for k in range(data_size_x):
                    alldata[:,j+offset_pixels_y,k+offset_pixels_x,i] = data_ifile[:,j,k]*scale_m[i] + scale_a[i]

    mask, boolean_mask = mask_bin(list_cubes, outfile, xoff, yoff, target_scale)       
    
    while_loop_count=0
    flux_calibrated_images_index = [0]
    while while_loop_count <= (len(list_cubes)-1):
        for i in range(len(list_cubes)-1):
            if np.isin(i+1,flux_calibrated_images_index) == False:
                npixels_solape = 0
                indice_solape_0 = []
                indice_solape_1 = []
                for m, valm in enumerate(flux_calibrated_images_index):
                    for j in range(boolean_mask.shape[0]):
                        for k in range(boolean_mask.shape[1]):                    
                            if boolean_mask[j,k][i+1] and boolean_mask[j,k][valm]:
                                indice_solape_0.append(j)
                                indice_solape_1.append(k)
                                npixels_solape+=1
                    if npixels_solape >= 10:
                        flux_calibrated_images_index.append(i+1)
                        flux_factor = sum(np.sum(alldata[400:3900,indice_solape_0,indice_solape_1,valm], axis=1))/sum(np.sum(alldata[400:3900,indice_solape_0,indice_solape_1,i+1], axis=1))
                        alldata[:,:,:,i+1] = flux_factor*alldata[:,:,:,i+1]
                        print(flux_factor, list_cubes[i+1])
                        print('Flux calibrated with', list_cubes[valm])
                        break
        while_loop_count+=1
        
    for i, ifile in enumerate(range(len(list_cubes))[::-1]):
        for j in range(hypercube_size_y):
            for k in range(hypercube_size_x):
                if np.mean(alldata[:,j,k,ifile]) > 0:
                    hypercube[:,j,k] = alldata[:,j,k,ifile]

    hdu[0].data = hypercube
    hdu.writeto(str(outfile), overwrite=True)


def main(args=None):
    import argparse
    import os
    import astropy.io.fits as fits

    # parse command-line options
    parser = argparse.ArgumentParser(prog='convert_rss_cube')
    # positional parameters
    methods = {'nn': 1, 'linear': 2}

    parser.add_argument("rss",
                        help="RSS file / List of RSS files",
                        type=argparse.FileType('rb'))
    parser.add_argument('-l', '--is-a-list', default=False, action="store_true", help='Use for -s being a list of FITS spectra')
    parser.add_argument('-c', '--is-a-cube', default=False, action="store_true", help='Use for -s being a list of cubes (not rss) spectra')
    parser.add_argument('-p', '--pixel-size', type=float, default=0.4,
                        metavar='PIXEL_SIZE',
                        help="Pixel size in arc seconds (default = 0.4)")
    parser.add_argument('-o', '--outfile', default='test',
                        help="Name of the output cube file (default = test")
    parser.add_argument('-d', '--disable-scaling', action='store_true',
                        help="Disable flux conservation")
    parser.add_argument('-m', '--method', action='store', choices=['nn', 'linear'],
                        default='nn', help="Method of interpolation")
    parser.add_argument('--wcs-pa-from-header', action='store_true',
                        help="Use PA angle from header", dest='pa_from_header')
    parser.add_argument('-trim', '--trimming', default=False, action="store_true", help='Use for trimming the cubes')
    parser.add_argument('-hyp', '--hyper', default=False, action="store_true", help='Use for creating the hypercube')
    parser.add_argument('-helio', '--helio', default=False, action="store_true", help='Use for applying heliocentric velocity correction')
    parser.add_argument('-trimn', '--trimming-numbers', nargs='*', default= [1,2,1,1], help='Use for declare the number of rows and columns you want to trim. [Bottom rows, top rows, left column, right column] (default= 1,2,1,1)')

    args = parser.parse_args(args=args)

    conserve_flux = not args.disable_scaling
    hexspline_order = methods[args.method]

    xoff = []
    yoff = []

    if args.is_a_list:
        xoff, yoff, scale_a, scale_m = np.loadtxt(args.rss, usecols=(1, 2, 3, 4), unpack=True)
        args.rss.seek(0)
        list_files = args.rss.readlines()
        list = [str(x.strip(),'utf-8') for x in list_files] 
        list_files = []
        for i in list:
            list_files.append(i.split()[0])
    else: 
        list_files = [args.rss.name]
        xoff.append(0.)
        yoff.append(0.)

    target_scale = args.pixel_size # Arcsec
    if not args.is_a_cube:
        print('\n', 'Creating cube from rss', '\n')
        print('target scale is', target_scale, 'arcsec')
        list_cubes = []
        for i in list_files:
#            cout = str(Path(i).parent) + './cube_' + str(Path(i).name)
            path = Path(i)
            new_path = path.parent / ('cube_' + path.name)
            cout = str(new_path)
            list_cubes.append(cout)
            rss = fits.open(i)

            rss_data = rss[0].data
            ############ Assigning an average value to the dead fibre from its six neighbouring fibres ############
            rss[0].data[622] = (rss_data[619] + rss_data[524] + rss_data[528] + rss_data[184] + rss_data[183] + rss_data[
                621]) / 6.0
            #######################################################################################################

            cube = mcube.create_cube_from_rss(rss, hexspline_order, target_scale, conserve_flux=conserve_flux)
            if not args.pa_from_header:
                print('recompute WCS from IPA')
                cube[0].header = mcube.recompute_wcs(cube[0].header)
            cube.writeto(cout,overwrite=True)

    else:
        list_cubes = list_files
        
    if args.trimming:
        print ('\n', 'Trimming cubes...', '\n')
        list_trimmed_cubes = []
        trim_numbers = args.trimming_numbers
        trim_numbers = [ int(x) for x in trim_numbers ]
        print(trim_numbers)
        for i, ifile in enumerate(list_cubes):
            path = Path(ifile)
            new_path = path.parent / ('trimmed_' + path.name)
            list_trimmed_cubes.append(str(new_path))
#            list_trimmed_cubes.append(str(Path(ifile).parent) + './trimmed_' + str(Path(ifile).name))
            trim_cubes(ifile, trim_numbers)
    
    if args.helio:
        helio_corr_apply = True
    else:
        helio_corr_apply = False
        
    if args.hyper:
        outfile = args.outfile
        print ('\n', 'Creating hypercube...', '\n')
        if args.trimming == True:
            grid_combined_cube(list_trimmed_cubes, helio_corr_apply, outfile, xoff, yoff, scale_a, scale_m, target_scale)
        else:
            grid_combined_cube(list_cubes, helio_corr_apply, outfile, xoff, yoff, scale_a, scale_m, target_scale)

if __name__ == '__main__':
    main()



#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

import argparse
import sys
import csv

import numpy as np
import random
import matplotlib.pyplot as plt
import matplotlib.transforms as transforms
from astropy.io import fits

from .analyze import axvlines

plt.rcParams.update({'font.size': 10})


def main(args=None):
# Parser
    parser = argparse.ArgumentParser(description='Input spectrum and table',prog='plot_spectrum')
    parser.add_argument('-s', '--spectrum', metavar='SPECTRUM/FILE_LIST', help='FITS spectrum / list of FITS spectra', type=argparse.FileType('rb'))
    parser.add_argument('-l', '--is-a-list', default=False, action="store_true", help='Use for -s being a list of FITS spectra')
    parser.add_argument('-t', '--std-table', metavar='STD-TABLE', help='Standard-star spectrum table', type=argparse.FileType('r'))
    parser.add_argument('-c', '--catalog', metavar='LINECAT-TABLE', help='Cataloged lines CSV table', type=argparse.FileType('r'))
    parser.add_argument('-z', '--redshift', metavar='LINECAT-Z', help='Redshift for catalog lines', type=float)
    parser.add_argument('-o', '--output', metavar='OUTPUT-PDF', help='Output PDF', type=argparse.FileType('w'))
    parser.add_argument('-e', '--efficiency', default=False, action="store_true" , help='Efficiency?')
    parser.add_argument('-p', '--plot', default=False, action="store_true" , help='Plot spectrum?')
    parser.add_argument('-n', '--no-legend', default=False, action="store_true", help='Legend?')
    parser.add_argument('-L1', '--min-lambda', metavar='INITIAL LAMBDA', help='Initial (rest-frame) lambda to plot', type=float)
    parser.add_argument('-L2', '--max-lambda', metavar='LAST LAMBDA', help='Last (rest-frame) lambda to plot', type=float)
    parser.add_argument('-F1', '--min-flambda', metavar='YMIN FLUX', help='Minimum flux to plot', type=float)
    parser.add_argument('-F2', '--max-flambda', metavar='YMAX FLUX', help='Maximum flux to plot', type=float)
    parser.add_argument('-T', '--title', metavar='PLOT TITLE', help='Title of the plot')

    args = parser.parse_args(args=args)

# Constants
    c_amstrong = 2.99792e+18  # Light speed in AA/s
    gtceffarea = 7.3e+5 # GTC effective area in cm**2
# Internal parameters
    plosses = 0.80 # Pupil losses
    tlosses = 0.80 # Telescope losses (relative to effective area)

    if args.redshift is not None:
       z=float(args.redshift)
    else:
       z=0.

# Plotting
    if args.std_table!=None or args.spectrum!=None:
       plt.figure()

    if args.spectrum!=None:   
# Reading spectrum/spectra
       if args.is_a_list == True:
          list_files = args.spectrum.readlines() 
          list_files = [str(x.strip(),'utf-8') for x in list_files] 
       else: 
          list_files = [args.spectrum.name] 
       for ifile in list_files:
          hdulist = fits.open(ifile)
          tbdata = hdulist[0].data
          prihdr = hdulist[0].header
          lambda0 = prihdr['CRVAL1']
          cdelt = prihdr['CDELT1']
          crpix = prihdr['CRPIX1']
          if ('VPH' in prihdr): 
             vph = prihdr['VPH'] 
          else:
             vph = 'LR'

          lambda_fin = lambda0 + (len(tbdata))*cdelt
       
          if 'BUNIT' in prihdr:
             bunit = prihdr['BUNIT']
          else:
             bunit = 'Jy' # To take into account that the QLA-generated 1D spectra have not BUNIT keyword
          if 'PIXLIMF1' in prihdr:
             pf1 = prihdr['PIXLIMF1'] # Sensitivity function computed in this region (beginning)
          else: 
             pf1 = crpix 
          if 'PIXLIMF2'in prihdr:
             pf2 = prihdr['PIXLIMF2'] # Sensitivity function computed in this region (end)
          else:
             pf2 = len(tbdata)
          if 'PIXLIMM1' in prihdr:
             pm1 = prihdr['PIXLIMM1'] # All fibers include this region (beginning)
          else:
             pm1 = crpix
          if 'PIXLIMM2' in prihdr:
             pm2 = prihdr['PIXLIMM2'] # All fibers include this region (end)
          else:
             pm2 = len(tbdata)

          flux = []
          wave = []
          for i in range(0,len(tbdata)):
              lambda_i = lambda0 + i*cdelt
              wave.append(lambda_i)
              if bunit == 'Jy':
                 flux_vector = (1e-23*tbdata[i]*c_amstrong)/(lambda_i**2)
              elif bunit == 'ELECTRON' or 'CGS' or 'cgs' in bunit:
                 if args.efficiency == True: 
                    eperjy = (gtceffarea*(1.51e+3/lambda_i))*cdelt # Photons to be collected /GTC/sec/pix/Jy
                    flux_vector = tbdata[i]/(eperjy*plosses*tlosses)
                 else:
                    flux_vector = tbdata[i]
              else:
                 print ('unknown or not defined BUNIT [Jy, ELECTRON]')
                 sys.exit(1)
              flux.append(flux_vector)
          lflines = [lambda0 + (pf1 - crpix) * cdelt, lambda0 + (pf2 - crpix) * cdelt]
          lmlines = [lambda0 + (pm1 - crpix) * cdelt, lambda0 + (pm2 - crpix) * cdelt]
          if "LR" in vph:
              plt.plot(wave, flux, 'blue', label = 'input spectrum')
          else: 
              if "MR" in vph:
                  plt.plot(wave, flux, 'green', label = 'input spectrum')
              else:
                  plt.plot(wave, flux, 'red', label = 'input spectrum')
          if (args.no_legend) == False:
              axvlines(lmlines, color='cyan', label = 'All-fiber range', linestyle = '-')
              axvlines(lflines, color='brown', label = 'Sensitivity range', linestyle = '--')

    if args.std_table!=None:
# Reading standard-star spectrum table
       xdat, ydat = np.loadtxt(args.std_table, skiprows=1, usecols=(0, 1), unpack=True)
       fydat = []
       for i in range(0,len(ydat)):
           flux_value = (1e-23*(3631.0*10.**(-0.4*ydat[i]))*c_amstrong)/(xdat[i]**2)
           fydat.append(flux_value)
       plt.plot(xdat, fydat, 'red', label = 'tabulated spectrum')

    if args.catalog!=None:
# Reading input line catalog
        with open(args.catalog.name, 'r') as csvfile:
            spamreader = csv.reader(csvfile, delimiter=',', quotechar='"')
            slines=[] # Wavelengths
            llines=[] # Labels
            ylines=[] # Y positions

            c=0
            a = np.array(list(spamreader))
            for i in a[:,1]:
                if ((args.min_lambda is None or (float(i) > (args.min_lambda*(1+z)))) and (args.max_lambda is None or (float(i) < (args.max_lambda*(1+z))))):
                    slines.append(float(i)*(1+z))
                    llines.append(str(a[c,0]))
                    ylines.append(random.uniform(0.9,0.98))
                c += 1
            axvlines(slines, color='green', label=None, linestyle = ':')
            trans = transforms.blended_transform_factory(plt.gca().transData, plt.gca().transAxes)
            for i in range(0,len(slines)):
                plt.text(slines[i],ylines[i],llines[i],transform=trans,fontsize='smaller')

# Plotting
    plt.title(args.title, fontsize='smaller')
    if args.spectrum!=None:
       if bunit == 'Jy' or 'CGS' or 'cgs' in bunit:
          plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ A$^{-1}$]')
       else:
          if args.efficiency == True:
             plt.ylabel('Efficiency')
          else:
             plt.ylabel('ELECTRON')
    elif args.std_table!=None:
         plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ A$^{-1}$]')

    if args.min_lambda != None:
        plt.xlim(left=((1+z)*args.min_lambda))
    if args.max_lambda != None:
        plt.xlim(right=((1+z)*args.max_lambda))
    if args.min_flambda != None:
        plt.ylim(bottom=args.min_flambda)
    if args.max_flambda != None:
        plt.ylim(top=args.max_flambda)

    if args.std_table!=None or args.spectrum!=None:
       plt.xlabel('wavelength')
       if args.no_legend == True:
          plt.legend('', frameon=False)
       else:
          plt.legend()
       if args.plot == True:
          plt.show()
       else:
          if args.output!=None:
             plt.savefig(args.output.name)
          else:
             plt.show()

if __name__ == '__main__':

    main()



#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

from __future__ import print_function
import numpy as np
import math
import sys
import re
import matplotlib.pyplot as plt

import numina.types.structured as structured

def main(args=None):
    from astropy.io import fits
    import argparse
    from megaradrp.datamodel import MegaraDataModel
    from matplotlib.backends.backend_pdf import PdfPages

# Parser
    parser = argparse.ArgumentParser(description='Cleaning of diffuse light from a reduced (non-RSS) MEGARA image',prog='clean_diffuse_light')
    parser.add_argument('-i', '--input', metavar='INPUT-IMAGE', help='Reduced FITS image', type=argparse.FileType('rb'))
    parser.add_argument('-o', '--output', metavar='OUTPUT-IMAGE', default='model.fits', help='Output diffuse-light FITS image', type=argparse.FileType('wb'))
    parser.add_argument('-r', '--residuals', metavar='RESIDUALS-IMAGE', default='residuals.fits', help='Output residual FITS image', type=argparse.FileType('wb'))
    parser.add_argument('-t', '--traces', metavar='MASTER-TRACES', help='Master traces JSON file', type=argparse.FileType('r'))
    parser.add_argument('-s', '--shift', metavar='SHIFT-TRACES', default=0., help='Traces shift', type=float)
    parser.add_argument('-w', '--window', metavar='SEARCH-WINDOW', default=6., help='Window around traces to search for non-illuminated fibers', type=float)
    parser.add_argument('-d', '--degree', metavar='DEGREE-POLY-COLS', default=4., help='Degree of polynomial fit for columns', type=float)
    parser.add_argument('-d2', '--degree-rows', metavar='DEGREE-POLY-ROWS', default=4., help='Degree of polynomial fit for rows', type=float)
    parser.add_argument('-p', '--outplot', metavar='OUTPUT-PLOT', help='Output plots', type=argparse.FileType('w'))
    parser.add_argument('-b', '--binning', metavar='SPECTRAL-BINNING', default=50, help='Binning in the spectral direction', type=int)
    parser.add_argument('-e', '--exclude', metavar='EXCLUDE-REGION', help='Exclude region (c1 c2 r1 r2), e.g. 2407 2720 0 164', nargs='+', type=int)
    parser.add_argument('-2D', '--two-dimensional', default=False, action="store_true", help='Two-dimensional fitting?')

    args = parser.parse_args(args=args)

    inputimage = args.input # Input reduced FITS image
    outputimage = args.output # Output diffuse-light FITS image
    resimage = args.residuals # Output residuals FITS image
    mtraces = args.traces
    otraces = args.shift

    hdulist = fits.open(inputimage)
    image_header = hdulist[0].header
    image2d = hdulist[0].data
 

    if args.exclude is not None:
        x1, x2, y1, y2 = args.exclude
        image2d[y1:y2,x1:x2] = np.NaN
    
    if 'naxis1' in image_header:
        naxis1 = image_header['naxis1']
    else:
        naxis1 = 4096
    if 'naxis2' in image_header:
        naxis2 = image_header['naxis2']
    else:
        naxis2 = 4112
    
    apers = structured.open(mtraces.name)

    columns = np.arange(0,naxis1)
    rows = np.arange(0,naxis2)
    model = np.empty_like(image2d)
    model2 = np.empty_like(image2d)
    bmodel = np.empty_like(image2d)
    
    if args.outplot!=None:
        pdf_pages = PdfPages(args.outplot.name)

    for i in columns[args.binning::args.binning]:
        fiberspos = [cc.polynomial(i) for cc in apers.contents if cc.valid]
        fluxes = np.nanmean(image2d[:,i-args.binning:i+args.binning],axis=1)

        lrows=[]
        for row in rows:
            for fibpos in fiberspos:
                if (abs(fibpos+args.shift-row) <= args.window):
                    lrows.append(row)
            if (np.isnan(fluxes[row])):
                lrows.append(row)
            
        frows=np.delete(rows,lrows)
        ffluxes=np.delete(fluxes,lrows)  
        plt.clf()
        plt.plot(rows,fluxes)
        plt.plot(frows,ffluxes,'.', color='black')
    
        bsize = min(i+args.binning,naxis1)-(i-args.binning)
        dfit=np.polyfit(frows,ffluxes,args.degree)
        pdfit = np.poly1d(dfit)
        plt.plot(rows,pdfit(rows))
        tmp = np.tile(pdfit(rows).reshape(naxis2,1),bsize)
        model[:,i-args.binning:i+args.binning]=tmp
        print('column:',i)
           
        plt.title('column: '+str(i+1))
        plt.xlabel('Y-position')
        plt.ylabel('Flux')
        if args.outplot!=None:
             pdf_pages.savefig()
        else:
             plt.show()

    if (args.two_dimensional):
        for i in rows[args.binning::args.binning]:
            fluxes = np.nanmean(model[i-args.binning:i+args.binning,:],axis=0)
            plt.clf()
            plt.plot(columns,fluxes)
            bsize = min(i+args.binning,naxis2)-(i-args.binning)
            dfit=np.polyfit(columns,fluxes,args.degree_rows)
            pdfit = np.poly1d(dfit)
            plt.plot(columns,pdfit(columns))
            tmp = np.transpose(np.tile(pdfit(columns).reshape(naxis1,1),bsize))
            model2[i-args.binning:i+args.binning,:]=tmp
            print('row:',i)

            plt.title('row: '+str(i+1))
            plt.xlabel('X-position')
            plt.ylabel('Flux')
            if args.outplot!=None:
                 pdf_pages.savefig()
            else:
                 plt.show()

    if args.outplot!=None:
        pdf_pages.close()
    plt.close()

    if (args.two_dimensional):
        bmodel = model2
    else:
        bmodel = model

    hdulist[0].data = bmodel
    hdulist.writeto(args.output.name, overwrite = True)
    hdulist[0].data = image2d - bmodel
    hdulist.writeto(args.residuals.name, overwrite = True)
    hdulist.close()

if __name__ == '__main__':

    main()




#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

from __future__ import print_function

import sys
import re

import numpy as np


def main(args=None):
    import argparse
    from astropy.io import fits
    import matplotlib.pyplot as plt
    import megaradrp.datamodel

    # Parser
    parser = argparse.ArgumentParser(description='Extract spectrum based on fiber IDs', prog='extract_spectrum')
    parser.add_argument(
        '-s', '--spectrum', metavar='RSS-SPECTRUM',
        help='RSS FITS spectrum', type=argparse.FileType('rb')
    )
    parser.add_argument(
        '-t', '--ids-table', metavar='INPUT-TABLE',
        help='File with list of IDs', type=argparse.FileType('r')
    )
    parser.add_argument(
        '-c', '--column', metavar='COLUMN',
        help='Column to select from table', type=int
    )
    parser.add_argument(
        '-g', '--grep-string', metavar='GREP-STRING',
        help='String to do grep in table'
    )
    parser.add_argument(
        '-o', '--output', metavar='OUTPUT-SPECTRUM',
        help='Output 1D spectrum', type=argparse.FileType('wb')
    )
    parser.add_argument(
        '-p', '--plot', default=False, action="store_true",
        help='Plot spectrum instead?'
    )

    args = parser.parse_args(args=args)

    spec = args.spectrum  # Input RSS FITS spectrum
    itable = args.ids_table  # Input File with list of IDs
    col = args.column-1  # Column to select from table
    gstring = args.grep_string  # String to do grep in table
    ofile = args.output  # Output 1D spectrum

# Reading spectrum
    rss = fits.open(spec)
    rss_data = rss[0].data
    prihdr = rss[0].header

    crval = prihdr['CRVAL1']
    crpix = prihdr['CRPIX1']
    cdelt = prihdr['CDELT1']

    fiberconf = megaradrp.datamodel.get_fiberconf(rss)
    connected = fiberconf.connected_fibers()
    rows = [conf.fibid - 1 for conf in connected]
    # Put in sellist all rows that need to be added
    # [fibid=row_in_sellist(that starts in zero)+1] -> row_in_sellist = fibid-1

    sellist = []
    for line in open(itable.name, 'r'):
        if re.search(gstring, line):
            sellist.append(int(line.split()[col])-1)
            if line is None:
                print('no matches found')
                sys.exit(1)

    region = rss_data[sellist, :]

    wave = []
    newima = np.sum(region, axis=0)
    for i in range(0, len(newima)):
        lambda_i = crval + i*cdelt
        wave.append(lambda_i)

    if args.plot:
        # Plot spectrum
        plt.figure()
        plt.plot(wave, newima, 'red', label='extracted spectrum')
        plt.legend()
        plt.xlabel('wavelength')
        plt.ylabel('flux [Jy]')
        plt.show()
    else:
        # Save spectrum
        ext_spectrum = fits.PrimaryHDU(newima, header=rss[0].header)
        ext_spectrum.header.remove('CRVAL2', ignore_missing=True, remove_all=True)
        ext_spectrum.header.remove('CRPIX2', ignore_missing=True, remove_all=True)
        ext_spectrum.header.remove('CDELT2', ignore_missing=True, remove_all=True)
        ext_spectrum.header.remove('CTYPE2', ignore_missing=True, remove_all=True)
        ext_spectrum.writeto(ofile, overwrite=True)

    #    spec = copy_img(rss)


if __name__ == '__main__':
    main()



#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

import argparse, textwrap
import sys
import csv

import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits
from lmfit import minimize, Parameters
from matplotlib.backends.backend_pdf import PdfPages

from .analyze import axvlines
from .analyze import gaussfunc, gauss2func, gaussfunc_gh
from .analyze import linfunc

def main(args=None):
# Parser
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,description=
        textwrap.dedent('''\
             __________________  ANALYZE_RSS PROGRAM  __________________\n
             ____________  OUTPUT PARAMETER IN OUTPUT FITS  ____________
             Property  channel description\n
        ...  FM        #  0 Fitting method (0=gauss-hermite,1=1gauss,2=2gauss)
        ...  CONTINUUM #  1 Continuum level in cgs
        ...  NOISE     #  2 rms in cgs
        ...  SNR       #  3 S/N at the peak of the line
        ...  FLUXD     #  4 Flux from window_data - window_continuum
        ...  EWD       #  5 Flux from window_data - window_continuum / mean_continuum
        ...  FLUXF     #  6 Flux from best-fitting function(s)
        ...  EWF       #  7 EW from best-fitting function(s)
        ...  H0        #  8 amplitude for methods 0 & 1 & 2 (first gaussian)
        ...  H1        #  9 central lambda for methods 0 & 1 & 2 (first gaussian)
        ...  H2        # 10 sigma (in AA) for methods 0 & 1 & 2 (first gaussian)
        ...  H3        # 11 h3 for method 0
        ...  H4        # 12 h4 for method 0
        ...  H0B       # 13 amplitude for method 2 (second gaussian)
        ...  H1B       # 14 central lambda for method 2 (second gaussian)  
        ...  H2B       # 15 sigma (in AA) for method 2 (second gaussian)
        ...  H1KS      # 16 velocity in km/s from H1 (1st g)
        ...  H2KS      # 17 sigma in km/s from H2 (1st g)
        ...  H2KLC     # 18 sigma in km/s from H2 corrected for instrumental sigma (1st g)
        ...  H1KSB     # 19 velocity in km/s from H1B (2nd g)
        ...  H2KSB     # 20 sigma in km/s from H2B (2nd g)
        ...  H2KLCB    # 21 sigma in km/s from H2 corrected for instrumental sigma (2nd g)
        ...  FLUXF1    # 22 Flux from best-fitting 1st gaussian 
        ...  FLUXF2    # 23 Flux from best-fitting 2nd gaussian 
        ...  EFLUXD    # 24 Error of 4 (Flux from window_data - window_continuum) 
        ...  EEWD      # 25 Error of 5 (Flux from window_data - window_continuum / mean_continuum)
        ...  EFLUXF    # 26 Error of 6 (Flux from best-fitting function(s))
        ...  EEWF      # 27 Error of 7 (EW from best-fitting function(s))
        ...  CHI2      # 28 best-fitting chi^2 (cgs)'''),
        prog='analyze_rss')
    
    parser.add_argument('-s', '--spectrum', metavar='RSS FILE', help='RSS input file', type=argparse.FileType('rb'))
    parser.add_argument('-f', '--method', default=0, choices=[0,1,2], metavar='FITTING FUNCTION (0,1,2)', help='Fitting function (0=gauss_hermite, 1=gauss, 2=double_gauss)', type=int)
    parser.add_argument('-S', '--limsnr', default=5, metavar='MINIMUM S/N', help='Mininum Signal-to-noise ratio in each spaxel', type=float)
    parser.add_argument('-w', '--ctwl', metavar='LINE CENTRAL WAVELENGTH', help='Central rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-k', '--use-peak', default=True, action="store_true", help='Use peak first guess on central wavelength')
    parser.add_argument('-LW1', '--lcut1', metavar='LOWER WAVELENGTH - LINE', help='Lower rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-LW2', '--lcut2', metavar='UPPER WAVELENGTH - LINE', help='Upper rest-frame wavelength for line (AA)',
                        type=float)
    parser.add_argument('-CW1', '--ccut1', metavar='LOWER WAVELENGTH - CONT', help='Lower rest-frame wavelength for cont. (AA)',
                        type=float)
    parser.add_argument('-CW2', '--ccut2', metavar='UPPER WAVELENGTH - CONT', help='Upper rest-frame wavelength for cont. (AA)',
                        type=float)
    parser.add_argument('-ECW1', '--eccut1', metavar='EXCLUDE FROM CONT. (LOWER WAVELENGTH)', help='Lower rest-frame wavelength of range to exclude for cont. (AA)',
                        type=float)
    parser.add_argument('-ECW2', '--eccut2', metavar='EXCLUDE FROM CONT. (UPPER WAVELENGTH)', help='Upper rest-frame wavelength of range to exclude for cont. (AA)',
                        type=float)
    parser.add_argument('-PW1', '--pcut1', metavar='LOWER WAVELENGTH - PLOT', help='Lower (observed) wavelength for plot (AA)',
                        type=float)
    parser.add_argument('-PW2', '--pcut2', metavar='UPPER WAVELENGTH - PLOT', help='Upper (observed) wavelength for plot (AA)',
                        type=float)
    parser.add_argument('-S2', '--scale-amp2', metavar='SCALE FACTOR FOR AMP2', help='Scale factor for amplitude 2', default=1.0, type=float)
    parser.add_argument('-z', '--redshift', metavar='REDSHIFT', help='Redshift for target and catalog lines', type=float)
    parser.add_argument('-o', '--output', metavar='OUTPUT-PDF', help='Output PDF', type=argparse.FileType('bw'))
    parser.add_argument('-v', '--verbose', default=False, action="store_true", help='Verbose mode for fitting results?')
    parser.add_argument('-O', '--output-rss', default="analyze_rss.fits", metavar='OUTPUT RSS FILE', help='Output RSS file', type=argparse.FileType('bw'))
    parser.add_argument('-of', '--output-fibers', metavar='OUTPUT FIBERS LIST', help='Output list of fibers above minimum Signal-to-noise ratio', type=argparse.FileType('bw'))


    args = parser.parse_args(args=args)
    
    plt.rcParams.update({'figure.max_open_warning': 0})
    
# Constants
    c_amstrong = 2.99792e+18  # Light speed in AA/s
    c_km = 2.99792e+5 # Light speed in km/s
    nonfit = float('NaN') # Value assigned to the out parameters when fit is not performed (could be 0. or NaN)
    zero = np.float64(0.0)

    if args.redshift is not None:
       z=float(args.redshift)
    else:
       z=0.

# Plotting

    if args.spectrum!=None:   
       ima = fits.open(args.spectrum)
       prihdr = ima[0].header
       lambda0 = prihdr['CRVAL1']
       cdelt = prihdr['CDELT1']
       crpix = prihdr['CRPIX1']
       vph = prihdr['VPH']
       ny = prihdr['NAXIS1']
       nx = prihdr['NAXIS2'] 
       rss = ima[0].data
    
       if 'BUNIT' in prihdr:
          bunit = prihdr['BUNIT']
       else:
          bunit = 'Jy' # To take into account that old QLA-generated 1D spectra might not BUNIT keyword
                
       if 'PIXLIMF1' in prihdr:
          pf1 = prihdr['PIXLIMF1'] # Sensitivity function computed in this region (beginning)
       else: 
          pf1 = crpix 
       if 'PIXLIMF2'in prihdr:
          pf2 = prihdr['PIXLIMF2'] # Sensitivity function computed in this region (end)
       else:
          pf2 = len(tbdata)
       if 'PIXLIMM1' in prihdr:
          pm1 = prihdr['PIXLIMM1'] # All fibers include this region (beginning)
       else:
          pm1 = crpix
       if 'PIXLIMM2' in prihdr:
          pm2 = prihdr['PIXLIMM2'] # All fibers include this region (end)
       else:
          pm2 = len(tbdata)

       plt.figure()
       plt.title('Collapsed spectrum')
       plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ $\AA$$^{-1}$]')
       plt.xlabel('wavelength')
        
       flux = []
       wave = []
       cdata = np.sum(rss, axis=0)

       for i in range(0,len(cdata)):
           lambda_i = lambda0 + i*cdelt
           wave.append(lambda_i)
           if bunit == 'Jy':
             flux_vector = (1e-23*cdata[i]*c_amstrong)/(lambda_i**2)
           elif bunit == 'ELECTRON' or 'CGS' or 'cgs' in bunit:
             flux_vector = cdata[i]
           else:
              print ('unknown or not defined BUNIT [Jy, ELECTRON]')
              sys.exit(1)
           flux.append(flux_vector)

       plt.xlim(lambda0,lambda0+float(len(cdata))*cdelt)

       if "LR" in vph:
            plt.plot(wave, flux, 'blue', label = 'input spectrum')
            R=6000.0
       else: 
            if "MR" in vph:
                 plt.plot(wave, flux, 'green', label = 'input spectrum')
                 R=12000.0
            else:
                 plt.plot(wave, flux, 'red', label = 'input spectrum')
                 R=20000.0

       lflines = [lambda0 + (pf1 - crpix) * cdelt, lambda0 + (pf2 - crpix) * cdelt]
       lmlines = [lambda0 + (pm1 - crpix) * cdelt, lambda0 + (pm2 - crpix) * cdelt]
       lllines = [float(args.lcut1)*(1.+z),float(args.lcut2)*(1.+z)]
       lclines = [float(args.ccut1)*(1.+z),float(args.ccut2)*(1.+z)]
       if (args.eccut1 is not None and args.eccut2 is not None):
        leclines = [float(args.eccut1) * (1. + z), float(args.eccut2) * (1. + z)]
       plines = [float(args.pcut1),float(args.pcut2)]
       cwline = [float(args.ctwl)*(1.+z)]
       axvlines(lmlines, color='cyan', label = 'All-fiber range', linestyle = '-')
       axvlines(lflines, color='brown', label = 'Sensitivity range', linestyle = '--')
       axvlines(cwline, color='black', label = 'Central wavelength', linestyle = '-')
       axvlines(lllines, color='gray', label = 'Line-fitting range', linestyle = '-')
       axvlines(lclines, color='gray', label = 'Continuum range', linestyle = '--')
       if (args.eccut1 is not None and args.eccut2 is not None):
        axvlines(leclines, color='gray', linestyle='--')
       axvlines(plines, color='green', label = 'Plot range', linestyle = '-.')
       plt.legend()

       plt.show()

       answer = input('Continue (Y/n)? ')
       if len(answer) is not 0 and answer[0].lower() == "n":
          sys.exit(1)
            
# Reading spectrum/spectra
       with PdfPages(args.output) as pdf:
           FM = []
           CONTINUUM = []
           NOISE = []
           SNR = []
           FLUXD = [] # Flux from window_data - window_continuum
           EWD = [] # Flux from window_data - window_continuum / mean_continuum
           EFLUXD = [] # Error of Flux from window_data - window_continuum
           EEWD = [] # Error of Flux from window_data - window_continuum / mean_continuum
           FLUXF = [] # Flux from best-fitting function
           EWF = [] # EW from best-fitting function
           EFLUXF = [] # Error of Flux from best-fitting function
           EEWF = [] # Error of EW from best-fitting function
           H0 = []
           H1 = []
           H2 = []
           H3 = []
           H4 = []
           H0B = []
           H1B = []
           H2B = []
           H1KS = []
           H2KS = []
           H2KLC = []
           H1KSB = []
           H2KSB = []
           H2KLCB = []     
           FLUXF1 = []
           FLUXF2 = []
           CHI2 = []
           FIB = [] # list of of fibers with SNR above input threshold
           
           for ispec in range(1,nx+1):
              FM.append(args.method)
              plt.figure()
              tbdata = rss[ispec-1,:]
              lambda_fin = lambda0 + (len(tbdata))*cdelt

              flux = []
              wave = []

              for i in range(0,len(tbdata)):
                  lambda_i = lambda0 + i*cdelt
                  wave.append(lambda_i)
                  if bunit == 'Jy':
                     flux_vector = (1e-23*tbdata[i]*c_amstrong)/(lambda_i**2)
                  elif bunit == 'ELECTRON' or 'CGS' or 'cgs' in bunit:
                     flux_vector = tbdata[i]
                  else:
                     print ('unknown or not defined BUNIT [Jy, ELECTRON]')
                     sys.exit(1)
                  flux.append(flux_vector)

              plt.xlim(float(args.pcut1),float(args.pcut2))

              if "LR" in vph:
                  plt.plot(wave, flux, 'blue', label = 'input spectrum')
                  R=6000.0
              else: 
                  if "MR" in vph:
                      plt.plot(wave, flux, 'green', label = 'input spectrum')
                      R=12000.0
                  else:
                      plt.plot(wave, flux, 'red', label = 'input spectrum')
                      R=20000.0

              fcont = []
              wcont = []
              fcont = flux[int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.ccut2)*(1.+z)-lambda0)/cdelt+crpix)]
#              wcont = wave[int((float(args.ccut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.ccut2)*(1.+z)-lambda0)/cdelt+crpix)]

# Excluding range
              if (args.eccut1 is not None and args.eccut2 is not None):
                  del fcont[int((float(args.eccut1) * (1. + z) - lambda0) / cdelt + crpix) - int(
                      (float(args.ccut1) * (1. + z) - lambda0) / cdelt + crpix):int(
                      (float(args.eccut2) * (1. + z) - lambda0) / cdelt + crpix) - int(
                      (float(args.ccut1) * (1. + z) - lambda0) / cdelt + crpix)]
              wcont = wave[int((float(args.ccut1) * (1. + z) - lambda0) / cdelt + crpix):int(
                  (float(args.ccut2) * (1. + z) - lambda0) / cdelt + crpix)]
              if (args.eccut1 is not None and args.eccut2 is not None):
                  del wcont[int((float(args.eccut1) * (1. + z) - lambda0) / cdelt + crpix) - int(
                      (float(args.ccut1) * (1. + z) - lambda0) / cdelt + crpix):int(
                      (float(args.eccut2) * (1. + z) - lambda0) / cdelt + crpix) - int(
                      (float(args.ccut1) * (1. + z) - lambda0) / cdelt + crpix)]
              cmean = np.mean(fcont)

# Fitting continuum
              p_lin = Parameters()
              p_lin.add('slope', value=zero, vary=True)
              p_lin.add('yord', value=cmean, vary=True)
              if (args.verbose):
                print("FITTING CONTINUUM:")
                print("Input(slope,yord):  %10.3E %10.3E" % (0., cmean))
              err_lin = lambda p, x, y: linfunc(p, x) - y
              fitout_lin = minimize(err_lin, p_lin, args=(wcont, fcont))
              fitted_p_lin = fitout_lin.params
              pars_lin = [fitout_lin.params['slope'].value, fitout_lin.params['yord'].value,fitout_lin.chisqr]
              if (args.verbose):
                  print("Output(slope,yord): %10.3E %10.3E" % (
              fitout_lin.params['slope'].value, fitout_lin.params['yord'].value))
                  print("Best-fitting chisqr continuum: %10.3E" % (fitout_lin.chisqr))
              fit_con = linfunc(fitted_p_lin, wcont)
              residuals = fcont - fit_con
              rms = np.std(residuals)

# Arrays for line profile

              fline = []
              fpline = []
              wline = []
              fline = flux[int((float(args.lcut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.lcut2)*(1.+z)-lambda0)/cdelt+crpix)]
              wline = wave[int((float(args.lcut1)*(1.+z)-lambda0)/cdelt+crpix):int((float(args.lcut2)*(1.+z)-lambda0)/cdelt+crpix)]
              peak = np.amax(fline)
              fit_lin = linfunc(fitted_p_lin, wline)
              lcmean = np.mean(fit_lin)  # Mean continuum within the line range
              fpline = fline - fit_lin
#              fpline = fline - cmean
              result = np.where(fline == np.amax(fline))
              lpeak = wline[result[0][0]]
            
              CONTINUUM.append(lcmean)
              NOISE.append(rms)
            
              if (np.isnan(peak/rms)):
                  peak=0.
                  rms=1E-31

              eEWd = (rms * cdelt * (cdelt * np.sum(fpline) / lcmean) / (cdelt * np.sum(fpline))) * np.sqrt(
                  2 * len(fpline) + np.sum(fpline) / lcmean + (np.sum(fpline) / lcmean) ** 2 / len(fpline))

              if np.isfinite(peak/rms) and (peak/rms) >= float(args.limsnr) and ispec!=623:
#                  print(ispec)
                  FIB.append(ispec)
                  SNR.append(peak/rms)
                  FLUXD.append(cdelt*np.sum(fpline))
                  EFLUXD.append(rms * cdelt * np.sqrt(2 * len(fpline) + (np.sum(fpline) / lcmean)))
                  EWD.append(cdelt*np.sum(fpline)/lcmean)
                  EEWD.append(eEWd)
              else:
                  SNR.append(nonfit)
                  FLUXD.append(nonfit)
                  EFLUXD.append(nonfit)
                  EWD.append(nonfit)
                  EEWD.append(nonfit)
                  
              if (args.verbose):
                  print("BASIC NUMBERS:")
                  print ("Fiber: %3d; Stat (mean,rms,lpk,pk,S/N): %10.3E %10.3E %5.2f %10.3E %5.2f"%(ispec,lcmean,rms,lpeak,peak,peak/rms))

    # Initial guess on parameters

              amp = peak-lcmean
              if (args.use_peak) == False:
                center = float(args.ctwl)*(1.+z)
              else:
                center = lpeak
              sigma = (float(args.ctwl)/R)*((1.+z)/2.35)
              skew = 0.0
              kurt = 0.0

              amp1 = 0.9*(peak-lcmean)
              sigma1 = (float(args.ctwl)/R)/2.35
              amp2 = 0.1*(peak-lcmean)
              sigma2 = 2.*(float(args.ctwl)/R)/2.35
              if (args.use_peak) == False:
                center1 = float(args.ctwl)*(1.+z)
                center2 = float(args.ctwl)*(1.+z)
              else:
                center1 = lpeak
                center2 = lpeak

              if args.method is 0:
                  p_gh=Parameters()
                  p_gh.add('amp',value=amp, vary=True);
                  p_gh.add('center',value=center, vary=True);
                  p_gh.add('sigma',value=sigma, vary=True, min=0.8*sigma);
                  p_gh.add('skew',value=skew,vary=True,min=-0.5,max=0.5);
                  p_gh.add('kurt',value=kurt,vary=True,min=-0.5,max=0.5);
                  if (args.verbose): 
                    print ("FITTING METHOD: GAUSS-HERMITE QUADRATURE")
                    print ("Input(i0,l0,sigma,skew,kurt):  %10.3E %5.2f %5.2f %10.3E %10.3E"%(amp, center, sigma, skew, kurt))
                  gausserr_gh = lambda p,x,y: gaussfunc_gh(p,x)-y 
              if args.method is 1:
                  p_gh=Parameters()
                  p_gh.add('amp',value=amp, vary=True);
                  p_gh.add('center',value=center, vary=True);
                  p_gh.add('sigma',value=1.2*sigma, vary=True, min=sigma);
                  if (args.verbose): 
                    print ("FITTING METHOD: SINGLE GAUSSIAN")
                    print ("Input(i0,l0,sigma):  %10.3E %5.2f %5.2f"%(amp, center, sigma))
                  gausserr_gh = lambda p,x,y: gaussfunc(p,x)-y
              if args.method is 2:
                  p_gh=Parameters()
                  p_gh.add('amp1',value=amp1, vary=True);
                  p_gh.add('center1',value=center1, vary=True);
                  p_gh.add('sigma1',value=sigma1, vary=True, min=0.8*sigma1);
                  p_gh.add('amp2',value=args.scale_amp2*amp2, vary=True);
                  p_gh.add('center2',value=center2, vary=True);
                  p_gh.add('sigma2',value=sigma2, vary=True, min=1.5*sigma1);
                  if (args.verbose): 
                    print ("FITTING METHOD: DOUBLE GAUSSIAN")
                    print ("Input(i1,l1,sig1,i2,l2,sig2):  %10.3E %5.2f %5.2f %10.3E %5.2f %5.2f"%(amp1, center1, sigma1, args.scale_amp2*amp2, center2, sigma2))
                  gausserr_gh = lambda p,x,y: gauss2func(p,x)-y

              fitout_gh=minimize(gausserr_gh,p_gh,args=(wline,fpline))
              fitted_p_gh = fitout_gh.params

              if args.method is 0:
                  pars_gh=[fitout_gh.params['amp'].value,fitout_gh.params['center'].value,fitout_gh.params['sigma'].value,fitout_gh.params['skew'].value,fitout_gh.params['kurt'].value,fitout_gh.chisqr]
                  fit_gh=gaussfunc_gh(fitted_p_gh,wline)
                  if (args.verbose): 
                    print ("Output(i0,l0,sigma,skew,kurt): %10.3E %5.2f %5.2f %10.3E %10.3E"%(fitout_gh.params['amp'].value, fitout_gh.params['center'].value, fitout_gh.params['sigma'].value, fitout_gh.params['skew'].value, fitout_gh.params['kurt'].value))
                    print('WARNING::::: out.covar == None :::::: Fit Sucess = '+ str(fitout_gh.success) + ' :: Uncertainties estimated = ' + str(fitout_gh.errorbars))

              if args.method is 1:
                  pars_gh=[fitout_gh.params['amp'].value,fitout_gh.params['center'].value,fitout_gh.params['sigma'].value,fitout_gh.chisqr]
                  fit_gh=gaussfunc(fitted_p_gh,wline)
                  if (args.verbose): 
                    print ("Output(i0,l0,sigma): %10.3E %5.2f %5.2f"%(fitout_gh.params['amp'].value, fitout_gh.params['center'].value, fitout_gh.params['sigma'].value))
                    print('WARNING::::: out.covar == None :::::: Fit Sucess = '+ str(fitout_gh.success) + ' :: Uncertainties estimated = ' + str(fitout_gh.errorbars))

              if args.method is 2:
                  pars_gh=[fitout_gh.params['amp1'].value,fitout_gh.params['center1'].value,fitout_gh.params['sigma1'].value,fitout_gh.params['amp2'].value,fitout_gh.params['center2'].value,fitout_gh.params['sigma2'].value,fitout_gh.chisqr] 
                  fit_gh=gauss2func(fitted_p_gh,wline)
                  if (args.verbose): 
                    print ("Output(i1,l1,sig1,i2,l2,sig2): %10.3E %5.2f %5.2f %10.3E %5.2f %5.2f"%(fitout_gh.params['amp1'].value, fitout_gh.params['center1'].value, fitout_gh.params['sigma1'].value, fitout_gh.params['amp2'].value, fitout_gh.params['center2'].value, fitout_gh.params['sigma2'].value))
                  tmp = fitted_p_gh
                  tmp.add('amp', value=tmp['amp1'].value)
                  tmp.add('center', value=tmp['center1'].value)
                  tmp.add('sigma', value=tmp['sigma1'].value)
                  fitted_p_gh1 = tmp
                  fit_gh1 = gaussfunc(fitted_p_gh1, wline)
                  if (args.verbose):
                      print("Flux1 from model: %10.3E+/-%10.3E" % (cdelt * np.sum(fit_gh1), rms * cdelt * np.sqrt(2 * len(fit_gh1) + (np.sum(fit_gh1) / lcmean))))  # Errors as in Tresse et al. (1999)
                  tmp2 = fitted_p_gh
                  tmp2.add('amp', value=tmp2['amp2'].value)
                  tmp2.add('center', value=tmp2['center2'].value)
                  tmp2.add('sigma', value=tmp2['sigma2'].value)
                  fitted_p_gh2 = tmp2
                  fit_gh2 = gaussfunc(fitted_p_gh2, wline)
                  if (args.verbose):
                      print("Flux2 from model: %10.3E+/-%10.3E" % (cdelt * np.sum(fit_gh2), rms * cdelt * np.sqrt(2 * len(fit_gh2) + (np.sum(fit_gh2) / lcmean))))  # Errors as in Tresse et al. (1999)

              eEWm = (rms * cdelt * (cdelt * np.sum(fit_gh) / lcmean) / (cdelt * np.sum(fit_gh))) * np.sqrt(
                  2 * len(fit_gh) + np.sum(fit_gh) / lcmean + (np.sum(fpline) / lcmean) ** 2 / len(fit_gh))
              if (args.verbose):
                  print ("Best-fitting chisqr: %10.3E"%(fitout_gh.chisqr))
                  print("Flux & EW from data:  %10.3E+/-%10.3E %5.2f+/-%5.2f" % (
                  cdelt * np.sum(fpline), rms * cdelt * np.sqrt(2 * len(fpline) + (np.sum(fpline) / lcmean)),
                  cdelt * np.sum(fpline) / lcmean, eEWd))  # Errors as in Tresse et al. (1999)
                  print("Flux & EW from model: %10.3E+/-%10.3E %5.2f+/-%5.2f" % (
                  cdelt * np.sum(fit_gh), rms * cdelt * np.sqrt(2 * len(fit_gh) + (np.sum(fit_gh) / lcmean)),
                  cdelt * np.sum(fit_gh) / lcmean, eEWm))  # Errors as in Tresse et al. (1999)
                  print("Best-fitting chisqr: %10.3E" % (fitout_gh.chisqr))

              # Here we write the results to a file and decide whether we should set them first to zero if S/N is below some threshold
              if (np.isnan(peak/rms)):
                  peak=0.
                  rms=1E-31

              if (np.isfinite(peak/rms)) and (peak/rms) >= float(args.limsnr) and ispec!=623:
                  FLUXF.append(cdelt*np.sum(fit_gh))
                  EFLUXF.append(rms * cdelt * np.sqrt(2 * len(fit_gh)))
                  EWF.append(cdelt*np.sum(fit_gh)/lcmean)
                  EEWF.append(eEWm)

                  plt.plot(wcont, fit_con, 'red', label='Continuum fit')
                  resid_gh=fpline-fit_gh
                  plt.plot(wline, fit_gh+fit_lin, 'orange', label = 'best fit')

                  lflines = [lambda0 + (pf1 - crpix) * cdelt, lambda0 + (pf2 - crpix) * cdelt]
                  lmlines = [lambda0 + (pm1 - crpix) * cdelt, lambda0 + (pm2 - crpix) * cdelt]
                  lllines = [float(args.lcut1)*(1.+z),float(args.lcut2)*(1.+z)]
                  lclines = [float(args.ccut1)*(1.+z),float(args.ccut2)*(1.+z)]
                  if (args.eccut1 is not None and args.eccut2 is not None):
                    leclines = [float(args.eccut1) * (1. + z), float(args.eccut2) * (1. + z)]
                  cwline = [float(args.ctwl)*(1.+z)]

                  axvlines(lmlines, color='cyan', label = 'All-fiber range', linestyle = '-')
                  axvlines(lflines, color='brown', label = 'Sensitivity range', linestyle = '--')
                  if (args.use_peak) == False:
                     axvlines(cwline, color='black', label = 'Central wavelength', linestyle = '-')
                  axvlines(lllines, color='gray', label = 'Line-fitting range', linestyle = '-')
                  axvlines(lclines, color='gray', label = 'Continuum range', linestyle = '--')
                  if (args.eccut1 is not None and args.eccut2 is not None):
                    axvlines(leclines, color='gray', linestyle='--')

                  if args.method is 0:       
                      H0.append(fitout_gh.params['amp'].value)
                      H1.append(fitout_gh.params['center'].value)
                      H2.append(fitout_gh.params['sigma'].value)
                      H3.append(fitout_gh.params['skew'].value)
                      H4.append(fitout_gh.params['kurt'].value)
                      H0B.append(nonfit)
                      H1B.append(nonfit)
                      H2B.append(nonfit)
                      H1KS.append(((fitout_gh.params['center'].value)/float(args.ctwl)-1.0)*c_km)
                      H2KS.append(((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)
                      if (np.isfinite(np.sqrt((((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))):
                         H2KLC.append(np.sqrt((((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))
                      else:
                         H2KLC.append(0.)
                      H1KSB.append(nonfit)
                      H2KSB.append(nonfit)
                      H2KLCB.append(nonfit)    
                      FLUXF1.append(nonfit)
                      FLUXF2.append(nonfit)
                  if args.method is 1:    
                      H0.append(fitout_gh.params['amp'].value)
                      H1.append(fitout_gh.params['center'].value)
                      H2.append(fitout_gh.params['sigma'].value)
                      H3.append(nonfit)
                      H4.append(nonfit)
                      H0B.append(nonfit)
                      H1B.append(nonfit)
                      H2B.append(nonfit)
                      H1KS.append(((fitout_gh.params['center'].value)/float(args.ctwl)-1.0)*c_km)
                      H2KS.append(((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)
                      if (np.isfinite(np.sqrt((((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))): 
                         H2KLC.append(np.sqrt((((fitout_gh.params['sigma'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))
                      else:
                         H2KLC.append(0.)
                      H1KSB.append(nonfit)
                      H2KSB.append(nonfit)
                      H2KLCB.append(nonfit)   
                      FLUXF1.append(nonfit)
                      FLUXF2.append(nonfit)
                  if args.method is 2:       
                      H0.append(fitout_gh.params['amp1'].value)
                      H1.append(fitout_gh.params['center1'].value)
                      H2.append(fitout_gh.params['sigma1'].value)
                      H3.append(nonfit)
                      H4.append(nonfit)
                      H0B.append(fitout_gh.params['amp2'].value)
                      H1B.append(fitout_gh.params['center2'].value)
                      H2B.append(fitout_gh.params['sigma2'].value)
                      H1KS.append(((fitout_gh.params['center1'].value)/float(args.ctwl)-1.0)*c_km)
                      H2KS.append(((fitout_gh.params['sigma1'].value)/float(args.ctwl))*c_km)
                      if (np.isfinite(np.sqrt((((fitout_gh.params['sigma1'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))): 
                         H2KLC.append(np.sqrt((((fitout_gh.params['sigma1'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))
                      else:
                         H2KLC.append(0.)
                      H1KSB.append(((fitout_gh.params['center2'].value)/float(args.ctwl)-1.0)*c_km)
                      H2KSB.append(((fitout_gh.params['sigma2'].value)/float(args.ctwl))*c_km)
                      if (np.isfinite(np.sqrt((((fitout_gh.params['sigma2'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2))):
                         H2KLCB.append(np.sqrt((((fitout_gh.params['sigma2'].value)/float(args.ctwl))*c_km)**2-(c_km/(R*2.35))**2)) 
                      else:
                         H2KLCB.append(0.)
                      s1 = fitout_gh.params['sigma1'].value
                      s2 = fitout_gh.params['sigma2'].value
                      FLUXF1.append(1.064*fitout_gh.params['amp1'].value*(2.35*s1))
                      FLUXF2.append(1.064*fitout_gh.params['amp2'].value*(2.35*s2))
                  CHI2.append(fitout_gh.chisqr)
              else:
                  FLUXF.append(nonfit)
                  EFLUXF.append(nonfit)
                  EWF.append(nonfit)
                  EEWF.append(nonfit)
                  H0.append(nonfit)
                  H1.append(nonfit)
                  H2.append(nonfit)
                  H3.append(nonfit)
                  H4.append(nonfit)
                  H0B.append(nonfit)
                  H1B.append(nonfit)
                  H2B.append(nonfit)
                  H1KS.append(nonfit)
                  H2KS.append(nonfit)
                  H2KLC.append(nonfit)
                  H1KSB.append(nonfit)
                  H2KSB.append(nonfit)
                  H2KLCB.append(nonfit)
                  FLUXF1.append(nonfit)
                  FLUXF2.append(nonfit)
                  CHI2.append(nonfit)

              plt.title('fiber: '+str(ispec))
              plt.ylabel('flux [erg s$^{-1}$ cm$^{-2}$ A$^{-1}$]')
              plt.xlabel('wavelength')
              plt.legend()
              pdf.savefig()
              plt.close()

# Storing output RSS file
       data1 = ima[0].data
       all_output = np.zeros([data1.shape[0],29])
        
# Assign each property to one of the 29 channels
       all_output[:,0] = FM        #  0 Fitting method (0=gauss-hermite,1=1gauss,2=2gauss)
       all_output[:,1] = CONTINUUM #  1 Continuum level in cgs
       all_output[:,2] = NOISE     #  2 rms in cgs
       all_output[:,3] = SNR       #  3 S/N at the peak of the line
       all_output[:,4] = FLUXD     #  4 Flux from window_data - window_continuum
       all_output[:,5] = EWD       #  5 Flux from window_data - window_continuum / mean_continuum
       all_output[:,6] = FLUXF     #  6 Flux from best-fitting function
       all_output[:,7] = EWF       #  7 EW from best-fitting function
       all_output[:,8] = H0        #  8 amplitude for methods 0 & 1 & 2 (first gaussian)
       all_output[:,9] = H1        #  9 central lambda for methods 0 & 1 & 2 (first gaussian)
       all_output[:,10] = H2       # 10 sigma (in AA) for methods 0 & 1 & 2 (first gaussian)
       all_output[:,11] = H3       # 11 h3 for method 0
       all_output[:,12] = H4       # 12 h4 for method 0
       all_output[:,13] = H0B      # 13 amplitude for method 2 (second gaussian)
       all_output[:,14] = H1B      # 14 central lambda for method 2 (second gaussian)
       all_output[:,15] = H2B      # 15 sigma (in AA) for method 2 (second gaussian)
       all_output[:,16] = H1KS     # 16 velocity in km/s from H1 (1st g)
       all_output[:,17] = H2KS     # 17 sigma in km/s from H2 (1st g)
       all_output[:,18] = H2KLC    # 18 sigma in km/s from H2 corrected for instrumental sigma (1st g)
       all_output[:,19] = H1KSB    # 19 velocity in km/s from H1B (2nd g)
       all_output[:,20] = H2KSB    # 20 sigma in km/s from H2B (2nd g)
       all_output[:,21] = H2KLCB   # 21 sigma in km/s from H2 corrected for instrumental sigma (2nd g)
       all_output[:,22] = FLUXF1   # 22 Flux from best-fitting 1st gaussian function
       all_output[:,23] = FLUXF2   # 23 Flux from best-fitting 2nd gaussian function
       all_output[:,24] = EFLUXD   # 24 Error of 4 (Flux from window_data - window_continuum)
       all_output[:,25] = EEWD     # 25 Error of 5 (Flux from window_data - window_continuum / mean_continuum)
       all_output[:,26] = EFLUXF   # 26 Error of 6 (Flux from best-fitting function)
       all_output[:,27] = EEWF     # 27 Error of 7 (EW from best-fitting function)
       all_output[:,28] = CHI2     # 28 best-fitting chi^2 (cgs)
    
       ima[0].data = all_output
       ima.writeto(args.output_rss, overwrite = True)

# Store the id of the fibers with SNR above minimum signal-to-noise ratio
       if args.output_fibers!=None: 
         np.savetxt(args.output_fibers, FIB, fmt='%d')

if __name__ == '__main__':

    main()




#
# Copyright 2019-2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#


def main(args=None):
    import shapely.geometry as sg
    import matplotlib.pyplot as plt
    from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator
    import numpy as np
    import shapely.affinity
    from astropy.io import fits
    import argparse
    import sys
    
# Parser
    parser = argparse.ArgumentParser(description='Extract spectra based on elliptical rings',prog='extract_elliptical_rings_spectrum')
    parser.add_argument('-r', '--rss', metavar='RSS-SPECTRUM', help='RSS FITS spectrum', type=argparse.FileType('rb'))
    parser.add_argument('-a', '--accumulate', default=False, action="store_true")
    parser.add_argument('-b', '--surface_brightness', default=False, action="store_true")
    parser.add_argument('-c', '--central-fiber', metavar='CENTRAL-FIBER', default=310, help='Central fiber', type=int) 
    parser.add_argument('-n', '--number-rings', metavar='NUMBER-RINGS', help='Number of rings', type=int) 
    parser.add_argument('-w', '--width', metavar='RINGS WIDTH', help='Elliptical rings width (arcsec)',type=float)
    parser.add_argument('-s', '--saved-rss', metavar='SAVED-RSS', help='Output RSS file', type=argparse.FileType('wb'))
    parser.add_argument('-e', '--ellipticity', metavar='ELLIPTICITY', help='Elliptical rings ellipticity',type=float)
    parser.add_argument('-pa', '--position-angle', metavar='POSITION ANGLE', help='Elliptical rings position angle (N->E)',type=float)
    parser.add_argument('-v', '--verbose', default=False, action="store_true")
    args = parser.parse_args(args=args)
  
    meg_spectra = args.rss
    fibra_central = args.central_fiber
    rings_number = args.number_rings

    plt.rcParams.update({'figure.max_open_warning': 0})

    hdu = fits.open(meg_spectra)
    gal_lin = hdu[0].data   ### Flux en Jy ###
    h1_0 = hdu[0].header
    locs_mm = hdu[1].header  ## en mm ##
    lam_gal = h1_0['CRVAL1'] + h1_0['CDELT1']*np.arange(h1_0['NAXIS1'])

    n=0
    b = np.zeros([gal_lin.shape[0],2])
    for i in range(gal_lin.shape[0]):
        if i < 9:
            b[n,0] = locs_mm[str('FIB00' + str(i+1) + '_X')]*1.212  ## El factor 1.212 es para pasar de mm a arcsec ##
            b[n,1] = locs_mm[str('FIB00' + str(i+1) + '_Y')]*1.212
        if 9 <= i < 99:
            b[n,0] = locs_mm[str('FIB0' + str(i+1) + '_X')]*1.212
            b[n,1] = locs_mm[str('FIB0' + str(i+1) + '_Y')]*1.212
        if 99 <= i < 999:
            b[n,0] = locs_mm[str('FIB' + str(i+1) + '_X')]*1.212
            b[n,1] = locs_mm[str('FIB' + str(i+1) + '_Y')]*1.212
        n+=1
    ell_x_offset = b[fibra_central-1][0]
    ell_y_offset = b[fibra_central-1][1]

    flux = 0
    if (args.surface_brightness):
        area_megara_spaxel = 3*np.sqrt(3)*(0.62/2)
    else:
        area_megara_spaxel = 1

    rings_data = np.zeros((rings_number,gal_lin.shape[1]))
    stacked_spectrum = np.zeros((gal_lin.shape[1]))
    total_area = 0
    
    amp_int=[]
    amp_ext=[]
    for i in range(rings_number):
        amp_int.append(float(args.width)*i)
        amp_ext.append(args.width+(args.width*i))
        
    fig = plt.figure(figsize=(8,8))
    for i in range(rings_number):
        for ind, val in enumerate(b):
            area = intersection(x_offset=val[0], y_offset=val[1],a_int=amp_int[i],a_ext=amp_ext[i],ellipticity=args.ellipticity,angle=90+args.position_angle,ell_x_offset=ell_x_offset,ell_y_offset=ell_y_offset)
            stacked_spectrum += gal_lin[ind]*area
            total_area += area
        rings_data[i] = stacked_spectrum/(total_area*area_megara_spaxel)
        if (args.verbose):
            print('Ring #', i+1, ': ', rings_data[i][2150], ' Jy/[asec/spx]^2 (@CWL) - area/rad: ',total_area*area_megara_spaxel,'/',(amp_ext[i]+amp_int[i])/2.,' [asec/spx]^2/asec)',sep="")
        if (args.accumulate is False):
            stacked_spectrum = np.zeros((gal_lin.shape[1]))
            total_area = 0
    hdu[0].data = rings_data
    hdu.writeto(args.saved_rss, overwrite = True)


def intersection(x_offset,y_offset,ell_x_offset,ell_y_offset,angle,a_ext,a_int,ellipticity):
    import matplotlib.pyplot as plt
    from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator
    import numpy as np
    from astropy.io import fits
    import descartes #it has to be installed
    import shapely.geometry as sg #it has to be installed
    import shapely.affinity #it has to be installed

    lado = 0.62/2
    vertices = [[x_offset+lado* 1/2,y_offset+lado*(np.sqrt(3)/2)],[x_offset+lado*1,y_offset+0],[x_offset+lado*1/2,y_offset-lado*(np.sqrt(3)/2)],
                           [x_offset-lado*1/2,y_offset-lado*(np.sqrt(3)/2)],[x_offset-lado*1,y_offset+0],[x_offset-lado*1/2,y_offset+(lado*np.sqrt(3)/2)]]
    hexagon = sg.Polygon(vertices)
    
    #### CENTRO DE LA ELIPSE ####
    centro = (ell_x_offset,ell_y_offset)
    
    ### ELIPSE exterior ###
    b_ext = a_ext*(1-ellipticity)
    ellipse_ext = (centro,(a_ext, b_ext),angle)
    circ_ext = sg.Point(ellipse_ext[0]).buffer(1)
    ell_ext  = shapely.affinity.scale(circ_ext, float(ellipse_ext[1][0]), float(ellipse_ext[1][1]))
    ellr_ext = shapely.affinity.rotate(ell_ext,ellipse_ext[2])
    interseccion_ext = hexagon.intersection(ellr_ext)
    inter_area_ext = interseccion_ext.area
    inter_area_ext_norm = inter_area_ext/hexagon.area
    
    ### ELIPSE interior ###
    b_int = a_int*(1-ellipticity)  #### ELIPTICIDAD CONSTANTE ####
    ellipse_int = (centro,(a_int, b_int),angle)
    circ_int = sg.Point(ellipse_int[0]).buffer(1)
    ell_int  = shapely.affinity.scale(circ_int, float(ellipse_int[1][0]), float(ellipse_int[1][1]))
    ellr_int = shapely.affinity.rotate(ell_int,ellipse_int[2])
    interseccion_int = hexagon.intersection(ellr_int)
    inter_area_int = interseccion_int.area
    inter_area_int_norm = inter_area_int/hexagon.area
    
    # use descartes to create the matplotlib patches
    ax = plt.gca()
    ax.add_patch(descartes.PolygonPatch(hexagon, fc='b', ec='w', alpha=0.2))
    ax.add_patch(descartes.PolygonPatch(ellr_ext.difference(ellr_int), fc='None', ec='k', alpha=0.05, zorder = 100))
    ax.add_patch(descartes.PolygonPatch(ellr_ext.difference(ellr_int), fc='r', ec='k', alpha=0.005, zorder = 100))    
    x_min = -6.2
    x_max = 6.2
    y_min = -6.2
    y_max = 6.2
    ax.set_xlim(x_min - abs(0.1*x_min), x_max + abs(0.1*x_max))
    ax.set_ylim(y_min - abs(0.1*y_min), y_max + abs(0.1*y_max))    
    ax.xaxis.set_tick_params(length = 5, width=1,labelsize=12)
    ax.yaxis.set_tick_params(length = 5, width=1,labelsize=12)
    plt.rc('xtick', direction = 'in')             
    plt.xlabel('[arcsec]', fontsize = 12)
    plt.ylabel('[arcsec]', fontsize = 12)  
    plt.setp(ax.spines.values(), linewidth=1,zorder=100)
    plt.subplots_adjust(left =0.1, bottom =0.2, right =0.9, top =0.99)
    ax.xaxis.set_minor_locator(AutoMinorLocator())
    ax.yaxis.set_minor_locator(AutoMinorLocator())
    ax.set_aspect('equal')    
    return inter_area_ext_norm-inter_area_int_norm

if __name__ == '__main__':
    main()




#
# Copyright 2020 Universidad Complutense de Madrid
#
# This file is part of Megara Tools
#
# SPDX-License-Identifier: GPL-3.0+
# License-Filename: LICENSE.txt
#

def main(args=None):
#    import math
    import astropy.io.fits as fits
    import argparse
    import numpy as np
        
    parser = argparse.ArgumentParser(description='Combining by averaging aligned RSS files',prog='combine_rss')
    parser.add_argument("rss",help="Input table with list of RSS files",type=argparse.FileType('rb'))
    parser.add_argument('-e', '--equation', metavar='Equation to evaluate', help='Example: \'(ima1[:,9] + ima2[:,10])/ ima3[:,3]\'')
    parser.add_argument('-o', '--output', default='combined_rss.fits', metavar='OUTPUT RSS', help='Output RSS', type=argparse.FileType('w'))

    args = parser.parse_args(args=args)

    list_files = args.rss.readlines() 
    list_files = [str(x.strip(),'utf-8') for x in list_files] 

    for x in range(0,len(list_files)):
        globals()['ima%s' % (x+1)] = fits.open(list_files[x])[0].data
    
    refima = fits.open(list_files[0])
    data1 = refima[0].data
    avgdata = np.zeros([data1.shape[0],1])
    avgdata[:,0] = eval(args.equation)
    
    refima[0].data = avgdata
    refima.writeto(args.output.name, overwrite = True)


# In[73]:

if __name__ == '__main__':

    main()
